#!/bin/sh
##############################################################################
# (C) Copyright 2012-14 Hewlett-Packard Development Company, L.P.
#     Confidential commercial computer software.
#
# 	      	
##############################################################################
# hadra.sh  - HP Serviceguard Sybase Toolkit DRA Module script
##############################################################################
#
# Version A.12.00.10
##############################################################################
# REVISION HISTORY:
#     	Version	        mm/dd/yyyy      Changes
#       01.00.00        04/10/2013      Sybase Toolkit on Linux 
#       12.00.00        11/29/2013      Added code for SG online modification failure check
#       12.00.10        07/18/2014      Removed redundant call to function validate_dra during
#                                       start up.
##############################################################################

CMD=${0}
#set usage display
Usage_Msg="Usage: ${CMD} {start | stop | validate | \
		| dra_monitor | reconfigure_stop | reconfigure_start}"


#####################################################
# Source cluster environment

if [[ -f /etc/cmcluster.conf ]]; then
    . /etc/cmcluster.conf
else
    /bin/echo "File /etc/cmcluster.conf doesn't exist"
    exit 1
fi

if [[ -z "${SG_SYBASE_MOD}" || ! ${SG_SYBASE_MOD} ]] ; then
# Source SG_UTILS & Sybase Toolkit global variables/functions
        . $SGROOT/sybasetoolkit/sybasetkit_utils.sh
fi

#########################################################################
#
#       Validation/Verification for supported actions
#
##########################################################################
# Verify whether unsupported toolkit attributes are being changed during
# online reconfiguration. If any of the unsupported attributes are being
# modified then function will return 1 (failure) else return 0 (success).
##########################################################################

function supported_online_modification
{
err_flag=false
typeset -i i

# attribute_name - List of attribute names for which online re-configure is not supported.
# cur_value - List of values of attributes, listed in (order of) attribute_name, while re-configuration.
# prev_value - List of values of attributes, listed in (order of) attribute_name, before re-configuration.

declare -a attribute_name=("dra/INSTALL_HOME" "dra/DRA_INSTALL_PATH" "dra/OCS_INSTALL_PATH" "dra/UNIQ_MONITOR_LABEL" "dra/ADMIN_NAME" "dra/LISTENER_PORT")
declare -a cur_value=("$DRA_INSTALL_HOME" "$DRA_DRA_INSTALL_PATH" "$DRA_OCS_INSTALL_PATH" "$DRA_UNIQ_MONITOR_LABEL" "$DRA_ADMIN_NAME" "$DRA_LISTENER_PORT")
declare -a prev_value=("$PREV_DRA_INSTALL_HOME" "$PREV_DRA_DRA_INSTALL_PATH" "$PREV_DRA_OCS_INSTALL_PATH" "$PREV_DRA_UNIQ_MONITOR_LABEL" "$PREV_DRA_ADMIN_NAME" "$PREV_DRA_LISTENER_PORT")

for (( i=0; i <= ${#attribute_name[@]} -1 ; i++ ))
do
  if [[ "${cur_value[i]}" != "${prev_value[i]}" ]]; then
      log_message 0 "ERROR: Online modification of the attribute ${attribute_name[i]} is not supported."
      err_flag=true
  fi
done

  if ${err_flag} ; then
      return 1
  fi

return 0
}

##Validate the package attributes for DR Agent module
function validate_dra
{
err_flag=false

log_message 5 "DRA Module verification started..."

# For online re-configuration
if [[ -z "$SG_PRIV_CLUSTER_VERIFICATION" || "$SG_COMMAND" = "cmapplyconf" ]]; then
    # Check if package is applied
    if $CMVIEWCL -f line -s config -p $SG_PACKAGE >$NULL 2>&1 ; then
    # Check if package is up, only then verify online modification of unsupported attributes
       if $CMVIEWCL -f line -p $SG_PACKAGE | $GREP -q -w "status=up" 2>$NULL ; then
          if [[ $OP_CODE != "start" ]] && [[ "${PREV_SG_MODULE_NAME[@]}" = *tkit/sybase/dra* ]] ; then
             if ! supported_online_modification ; then
                return 1
             fi
          fi 
       fi
    fi
fi


# Check for Linux user
if ! $ID $DRA_ADMIN_NAME >$NULL 2>&1 ; then
    log_message 0 "ERROR: DRA admin user $DRA_ADMIN_NAME does not exist"
    log_message 0 " - Add user $DRA_ADMIN_NAME on the node ${HOST}."
    err_flg=true
fi

# Confirm that DRA_HEALTHCHK_CMD variable contains non-null string
if [[ -z "$DRA_HEALTHCHK_CMD" && $OP_CODE != "stop" ]] ; then
    log_message 0 "WARNING: dra/HEALTHCHK_CMD is not defined in package configuration"
    log_message 0 " - DRA health check will not be performed."
fi

# Confirm that DRA_HEALTHCHK_RESULT variable contains non-null string
if [[ -n $DRA_HEALTHCHK_CMD && -z "$DRA_HEALTHCHK_RESULT" && $OP_CODE != "stop" ]] ; then
    log_message 0 "ERROR: dra/HEALTHCHK_RESULT is not defined for the health check command \"${DRA_HEALTHCHK_CMD}\"."
    err_flag=true
elif [[ -n $DRA_HEALTHCHK_CMD && -n "$DRA_HEALTHCHK_RESULT" && $OP_CODE != "stop" ]] ; then
    log_message 0 "Configuring dra/HEALTHCHK_CMD:\"${DRA_HEALTHCHK_CMD}\" and dra/HEALTHCHK_RESULT:\"${DRA_HEALTHCHK_RESULT}\" "
fi

DRA_SHARED=false
if [[ -n "${SG_FS_DIRECTORY[@]}" ]] ; then
for (( i=0 ; i < ${#SG_FS_DIRECTORY[@]} ; i++ )) ; do
     if $ECHO "${DRA_INSTALL_HOME}" | $GREP -q -w "${SG_FS_DIRECTORY[$i]}" 2>$NULL ; then
        DRA_SHARED=true
     fi
done
fi

# Non-Shared & Repserver module exist/Startup check
# If a DR Agent is packaged seperately, and has dependency mentioned on Repserver then we might 
# have to discard following verification as it might end up giveing error. VGs will not be mentioned 
# in DR Agent package configuration in that case! 
if ( ! ${DRA_SHARED} && ${REPSERV} ) || [[ $OP_CODE = "start" ]] 
then

   if [[ ! -d $DRA_INSTALL_HOME ]] ; then
       log_message 0 "ERROR: dra/INSTALL_HOME - $DRA_INSTALL_HOME directory does not exist."
       err_flag=true
       return 1
   fi

   if [[ ! -d $DRA_DRA_INSTALL_PATH ]] ; then
       log_message 0 "ERROR: dra/DRA_INSTALL_PATH - $DRA_DRA_INSTALL_PATH directory does not exist."
       err_flag=true
   fi

   if [[ ! -d $DRA_OCS_INSTALL_PATH ]] ; then
       log_message 0 "ERROR: dra/OCS_INSTALL_PATH - $DRA_OCS_INSTALL_PATH  directory does not exist."
       err_flag=true
   fi

   if [[ ! -f $DRA_PASSWD_FILE ]] ; then
       log_message 0 "ERROR: Specified dra/PASSWD_FILE - $DRA_PASSWD_FILE does not exist"
       log_message 0 " - Create the file $DRA_PASSWD_FILE with appropriate password in it," 
       log_message 0 "   otherwise, graceful shutdown or health check will not be performed."
       err_flag=true
   fi

   if [[ ! -f ${DRA_RUNSCRIPT} && $OP_CODE != "stop" ]] ; then
       log_message 0 "ERROR: dra/RUNSCRIPT - ${DRA_RUNSCRIPT} does not exist,"
       log_message 0 " - Provide the run script's path and name appropriately. "
       err_flag=true
   fi

   if  [[ ! -x $DRA_OCS_INSTALL_PATH/bin/isql ]] ; then
        log_message 0 "ERROR: Executable - \"$DRA_OCS_INSTALL_PATH/bin/isql\" does not exist"
        log_message 0 " - Make sure that isql binary is present in directory - $DRA_OCS_INSTALL_PATH/bin"
        err_flg=true
   fi

fi #end of Non-Shared/Startup check

if ${err_flag} ; then
	log_message 5 "DRA Module verification failed..."
	return 1
fi

log_message 5 "DRA Module verification succeeded..."
return 0
}


function start_dra
{

$PS -aef | $GREP ${DRA_UNIQ_MONITOR_LABEL} | $GREP ${DRA_MONITOR_PROCESS[0]} | $GREP -v "$GREP" >$NULL 2>&1
if [[ $? -eq 0 ]] ; then
    log_message 0 "${MODULE}: DR Agent Process ${DRA_MONITOR_PROCESS[0]} is already started"
else

   $SU - ${DRA_ADMIN_NAME} -c "/bin/sh" << STARTDRA
   if [[ -f ${DRA_INSTALL_HOME}/SYBASE.sh ]] ; then
      . ${DRA_INSTALL_HOME}/SYBASE.sh
      export PATH=$PATH:${DRA_DRA_INSTALL_PATH}/bin
      ${DRA_RUNSCRIPT} >> ${DRA_CONSOLE_LOGFILE} 2>&1 &
      $SLEEP ${DRA_TIME_OUT}
   else
      $ECHO "ERROR: ${MODULE}: Sybase environment variable file - ${DRA_INSTALL_HOME}/SYBASE.sh is missing."
   fi
STARTDRA

## Check if process has started successfully - using ${DRA_RETRY_TIME}
   $PS -aef | $GREP ${DRA_UNIQ_MONITOR_LABEL} | $GREP ${DRA_MONITOR_PROCESS[0]} | $GREP -v "$GREP" >$NULL 2>&1
   if [[ $? -eq 0 ]] ; then
      log_message 0 "${MODULE}: DR Agent process ${DRA_MONITOR_PROCESS[0]} started successfully"
   else
      log_message 0 "ERROR: ${MODULE}: process ${DRA_MONITOR_PROCESS[0]} could not be started"
      return 1
   fi

fi
return 0
}

function stop_dra
{
err_flag=false
declare -a DRA_PID
CLUSTER_IP=${HOST}
typeset -i retry_kill=1

if ! ( $PS -aef | $GREP ${DRA_UNIQ_MONITOR_LABEL} | $GREP ${DRA_MONITOR_PROCESS[0]} | $GREP -v -w grep >$NULL 2>&1 ) ; then
   log_message 0 "DRA: Process ${DRA_MONITOR_PROCESS[0]} is already stopped/aborted."
else
  if [[ -n ${DRA_PASSWD_FILE} && -f ${DRA_PASSWD_FILE} ]] ; then
     #get password from file
     if PASSWD=`$HEAD -n 1 ${DRA_PASSWD_FILE} 2>$NULL | $AWK '{print $1}' 2>$NULL ` ; then
        if [[ -z "${PASSWD}" ]] ; then
            log_message 0 "WARNING: ${MODULE}: No password in file ${DRA_PASSWD_FILE}."
        else
            $SU - ${DRA_ADMIN_NAME} -c "/bin/sh" << STOPDRA
            if [[ -f ${DRA_INSTALL_HOME}/SYBASE.sh ]] ; then
                . ${DRA_INSTALL_HOME}/SYBASE.sh
                export PATH=$PATH:${DRA_DRA_INSTALL_PATH}/bin
${DRA_OCS_INSTALL_PATH}/bin/isql -U${DRA_USER_NAME} -P${PASSWD} -S${CLUSTER_IP}:${DRA_LISTENER_PORT} << SHUTDWN >$NULL
shutdown
go

SHUTDWN
               $SLEEP ${DRA_TIME_OUT}
            else
               $ECHO "ERROR: DRA: Sybase environment variable file - ${DRA_INSTALL_HOME}/SYBASE.sh is missing."
            fi
STOPDRA
        fi #end of if PASSWD null
     else
        log_message 0 "WARNING: ${MODULE}: Could not get password from file ${DRA_PASSWD_FILE}."
     fi #end of get password from file
  else
     log_message 0 "WARNING: ${MODULE}: Password file - ${DRA_PASSWD_FILE} does not exist"
     log_message 3 " - DRA process will be aborted." 
  fi

fi # end of DRA process exist
 
# Forceful shutdown
i=0
while [[ ${DRA_MONITOR_PROCESS[@]} != "" ]]
do

   DRA_PID[${i}]=`$PS -efa | $GREP ${DRA_UNIQ_MONITOR_LABEL} | $GREP ${DRA_MONITOR_PROCESS[${i}]} | $GREP -v grep  | $AWK '{print $2}' 2>$NULL`

   if [[ -n ${DRA_PID[${i}]} ]]   
   then
      if $KILL -9 ${DRA_PID[${i}]} 2>/$NULL ; then  
         log_message 5 "${MODULE}: Success in killing ${DRA_MONITOR_PROCESS[$i]}, PID: ${DRA_PID[${i}]}"
         unset DRA_MONITOR_PROCESS[$i]
         i=`expr $i + 1`
      else
         if (( $retry_kill >= ${DRA_RETRY_TIME} ))
         then
            log_message 0 "ERROR: ${MODULE}: Process ${DRA_MONITOR_PROCESS[${i}]}, PID:${DRA_PID[${i}]} could not be stopped"
            sg_send_mail "${HOST}: Could not stop/kill ${DRA_MONITOR_PROCESS[${i}]} in ${SG_PACKAGE}" &&
            $SLEEP 3
            unset DRA_MONITOR_PROCESS[$i]
            i=`expr $i + 1`
            err_flag=true
            retry_kill=1
         fi
         retry_kill=`expr $retry_kill + 1`
      fi
  else 
     log_message 0 "${MODULE}: Process ${DRA_MONITOR_PROCESS[${i}]} has been stopped successfully."
     unset DRA_MONITOR_PROCESS[$i]
     i=`expr $i + 1`
  fi

done
if  ${err_flag} ; then
    log_message 0 "ERROR: ${MODULE}: All processes are not stopped"
    return 1
fi

log_message 0 "${MODULE}: All processes are stopped."
return 0
}

##########################################################################
#
#       Re-configure Start/Stop functions
#
##########################################################################
# verify_service_restart
# return 0 - At least one attribute changed which needs service restart.
# return 1 - No attribute that needs restart has been changed.
# Based on, whether supported attribute changed which needs service restart
# on change, decide upon the restart of the service
##########################################################################

function verify_service_restart
{
#Online re-configuration done to email attribute?
var1=( $(
     for id in "${SG_EMAIL_ID[@]}"
     do
       $ECHO "$id"
     done | $SORT ) )

var2=( $(
     for id in "${PREV_SG_EMAIL_ID[@]}"
     do
       $ECHO "$id"
     done | $SORT ) )

if [[ "${var1[@]}" != "${var2[@]}" ]] ; then
    return 0
fi

#Online re-configuration done to MONITOR_PROCESS ?
var3=( $(
     for id in "${DRA_MONITOR_PROCESS[@]}"
     do
       $ECHO "$id"
     done | $SORT ) )

var4=( $(
     for id in "${PREV_DRA_MONITOR_PROCESS[@]}"
     do
       $ECHO "$id"
     done | $SORT ) )

if [[ "${var3[@]}" != "${var4[@]}" ]] ; then
    return 0
fi

if [[ "$SG_SCRIPT_LOG_FILE" != "$PREV_SG_SCRIPT_LOG_FILE" ]] ; then
	return 0
fi

#Any other online reconfiguration supported attributes have been changed
#which needs restart of service?

if [[ "$DRA_MONITOR_INTERVAL" != "$PREV_DRA_MONITOR_INTERVAL" ]] ||
[[ "$DRA_USER_NAME" != "$PREV_DRA_USER_NAME" ]] ||
[[ "$DRA_PASSWD_FILE" != "$PREV_DRA_PASSWD_FILE" ]] ||
[[ "$DRA_RETRY_TIME" != "$PREV_DRA_RETRY_TIME" ]] ||
[[ "$DRA_HEALTHCHK_CMD" != "$PREV_DRA_HEALTHCHK_CMD" ]] ||
[[ "$DRA_HEALTHCHK_RESULT" != "$PREV_DRA_HEALTHCHK_RESULT" ]] ; then
    return 0
fi


#There were no attribute re-configuration done
#which would have required service restart

 log_message 5 "${MODULE}: None of the attribute that needs service restart have been modified online."
return 1
}


function reconfigure_stop_function
{
if ! sg_check_if_reconfig_has_failed ; then
   if verify_service_restart ; then
      if ! ${SGSBIN}/cmhaltserv ${service_name} ; then
         log_message 999 "ERROR: Could not stop service: ${service_name}"
         sg_set_reconfig_failure
         return 1
      else
         log_message 999 "SUCCESS: Successfully stopped service: $service_name"
         log_message 0 "SUCCESS: Successfully stopped service: $service_name" 
      fi
   fi
else 
   sg_log 999 "WARNING: Not attempting to stop the service: ${service_name}"  
fi

return 0
}

function reconfigure_start_function
{
   if verify_service_restart ; then
      msg="cmrunserv: Unable to start service $service_name (it is already active)"
      out=`${SGSBIN}/cmrunserv ${service_name} "${service_cmd_string}" 2>&1`
      status=$?
      if [[ $status -ne 0 ]] && [[ "$msg" != "$out" ]] ; then
         log_message 999 "ERROR: Failed to start the service $service_name after online package modification."
         sg_set_reconfig_failure
         return 1
      elif [[ $status -eq 0 ]] ; then
         log_message 999 "SUCCESS: Successfully started service: $service_name"
            if ! sg_check_if_reconfig_has_failed ; then
               log_message 0 "SUCCESS: Successfully started service: $service_name"
            fi         
      fi
   fi
return 0
}

################################### MAIN #################################

exit_val=0
OP_CODE=$1
DRAGENT=true
MODULE=DRA
CLUSTER_IP=${SG_IP_ADDRESS:-$HOST}

log_message 5 "Module hadra.sh is called with argument(s): $*"

#Identify the service name and service command for the package
if [[ $OP_CODE = "reconfigure_stop" || $OP_CODE = "reconfigure_start" ]]; then
    if [[ ${#SG_SERVICE_CMD[@]} -ne 0 ]]; then
        i=0
        service_name=""
        service_cmd_string=""
        for cmd in "${SG_SERVICE_CMD[@]}"
        do
         $ECHO "$cmd" | $GREP -q -w "dra_monitor" >$NULL 2>&1
          if [[ $? -eq 0 ]]
          then
              service_name=${SG_SERVICE_NAME[$i]}
              service_cmd_string="$cmd"
              break
          fi
          i=`expr $i + 1`
        done
     fi
fi

case $OP_CODE in
   start)
        if sg_check_if_reconfig_has_failed ; then
           log_message 999 "WARNING: Not attempting to start Sybase DRA module"
           return
        else
            if ! start_dra ; then
               if sg_check_if_it_is_reconfiguration ; then
                  log_message 999 "ERROR: Failed to start Sybase DRA module"
                  sg_set_reconfig_failure
               fi
               check_return 1 1
            else
                 if sg_check_if_it_is_reconfiguration ; then
                    log_message 999 "SUCCESS: Successfully started Sybase DRA module"
                 fi
            fi
        fi

       ;;
   stop)
        if ! stop_dra ; then
             check_return 1 2
        fi
       ;;
   validate)
        if ! validate_dra ; then
             check_return 1 4
        fi
       ;;

   dra_monitor)
        . ${TKIT_SCRIPT_DIR}/hadra.mon
             check_return $? 3
        ;;

   reconfigure_stop)
         if ! reconfigure_stop_function ; then
             log_message 0 "${MODULE}: Service stop failed for - $service_name"
             exit 1
         fi
	;;

   reconfigure_start)
         if ! reconfigure_start_function ; then
            log_message 0 "${MODULE}: Service start failed for - $service_name"
            exit 1
         fi
	;;
    *)
        log_message 0 "${Usage_Msg}"
	exit_val=1
        ;;
esac

log_message 5 "End of hadra.sh module"

exit ${exit_val}
