#!/bin/sh
##############################################################################
# (C) Copyright 2012-14 Hewlett-Packard Development Company, L.P.
#     Confidential commercial computer software.
#
# 	      	
##############################################################################
# hars.sh  -   HP Serviceguard Sybase Toolkit RS Module script
#
# Version A.12.00.10
##############################################################################
# REVISION HISTORY:
#       Version	        mm/dd/yyyy      Changes
#       01.00.00        04/10/2013      Sybase Toolkit on Linux - New release
#       12.00.00        11/29/2013      Added code for SG online modification failure check
#       12.00.10        07/18/2014      Removed redundant call to function validate_rs during
#                                       start up.
##############################################################################

typeset CMD=${0}
typeset RSSD_SERVER_NAME
typeset Usage_Msg="Usage: ${CMD} {start | stop | validate | rs_monitor \
		| reconfigure_stop | reconfigure_start}"

#####################################################
# Source cluster environment
if [[ -f /etc/cmcluster.conf ]]; then
    . /etc/cmcluster.conf
else
    /bin/echo "File /etc/cmcluster.conf doesn't exist"
    exit 1
fi

if [[ -z "${SG_SYBASE_MOD}" || ! ${SG_SYBASE_MOD} ]] ; then
# Source SG_UTILS & Sybase Toolkit global variables/functions
    . $SGROOT/sybasetoolkit/sybasetkit_utils.sh
fi

#########################################################################
#
#	Validation/Verification for supported actions
#
##########################################################################
# Verify whether unsupported toolkit attributes are being changed during		
# online reconfiguration. If any of the unsupported attributes are being		
# modified then function will return 1 (failure) else return 0 (success).
##########################################################################
function supported_online_modification
{
err_flag=false
typeset -i i

# attribute_name - List of attribute names for which online re-configure is not supported.
# cur_value - List of values of attributes, listed in (order of) attribute_name, while re-configuration.
# prev_value - List of values of attributes, listed in (order of) attribute_name, before re-configuration.

declare -a attribute_name=("rs/INSTALL_HOME" "rs/RS_INSTALL_PATH" "rs/OCS_INSTALL_PATH" "rs/INSTANCE_NAME" "rs/ADMIN_NAME" "rs/LISTENER_PORT")
declare -a cur_value=("$RS_INSTALL_HOME" "$RS_RS_INSTALL_PATH" "$RS_OCS_INSTALL_PATH" "$RS_INSTANCE_NAME" "$RS_ADMIN_NAME" "$RS_LISTENER_PORT")
declare -a prev_value=("$PREV_RS_INSTALL_HOME" "$PREV_RS_RS_INSTALL_PATH" "$PREV_RS_OCS_INSTALL_PATH" "$PREV_RS_INSTANCE_NAME" "$PREV_RS_ADMIN_NAME" "$PREV_RS_LISTENER_PORT")

for (( i=0; i <= ${#attribute_name[@]} -1 ; i++))
do
  if [[ "${cur_value[i]}" != "${prev_value[i]}" ]]; then
      log_message 0 "ERROR: Online modification of the attribute ${attribute_name[i]} is not supported."
      err_flag=true
  fi
done

  if ${err_flag} ; then
      return 1
  fi

return 0
}
 
#Validate the package attributes for RS module
function validate_rs
{
err_flg=false

log_message 5 "RS Module verification started."

if [[ -z "$SG_PRIV_CLUSTER_VERIFICATION" || $SG_COMMAND = "cmapplyconf" ]]; then
    # Check if package is applied
    if $CMVIEWCL -f line -s config -p $SG_PACKAGE >$NULL 2>&1 ; then

    # Check if package is up, only then verify online modification of unsupported attributes
       if $CMVIEWCL -f line -p $SG_PACKAGE | $GREP -q -w "status=up" 2>$NULL ; then
          if [[ $OP_CODE != "start" ]] && [[ "${PREV_SG_MODULE_NAME[@]}" = *tkit/sybase/rs* ]] ; then
             if ! supported_online_modification ; then
                return 1
             fi
          fi 
       fi
    fi
fi

RS_SHARED=false 
if [[ -n "${SG_FS_DIRECTORY[@]}" ]] ; then
for (( i=0; i<${#SG_FS_DIRECTORY[@]}; i++)) ; do
     if $ECHO $RS_INSTALL_HOME | $GREP -q -w ${SG_FS_DIRECTORY[$i]} 2>$NULL ; then
        RS_SHARED=true
     fi
done
fi

if ! ${RS_SHARED} || [[ $OP_CODE = "start" ]] 
then
    if [[ ! -d $RS_INSTALL_HOME ]] ; then
        log_message 0 "ERROR: rs/INSTALL_HOME - $RS_INSTALL_HOME directory does not exist."
        err_flg=true
        return 1
    fi

    if [[ ! -d $RS_RS_INSTALL_PATH ]] ; then
        log_message 0 "ERROR: rs/RS_INSTALL_PATH - $RS_RS_INSTALL_PATH directory does not exist."
        err_flg=true
    fi

    if [[ ! -d $RS_OCS_INSTALL_PATH ]] ; then
        log_message 0 "ERROR: rs/OCS_INSTALL_PATH - $RS_OCS_INSTALL_PATH directory does not exist."
        err_flg=true
    fi

    if [[ ! -f $RS_PASSWD_FILE ]] ; then
        log_message 0 "ERROR: rs/PASSWD_FILE - $RS_PASSWD_FILE does not exist."
        log_message 0 " - Create the file $RS_PASSWD_FILE with appropriate password in it,"
        log_message 0 "otherwise, health check or graceful shutdown will not be performed."
        err_flg=true
    fi

    if [[ ! -f $RS_RUNSCRIPT && $OP_CODE != "stop" ]] ; then
        log_message 0 "ERROR: rs/RUNSCRIPT - $RS_RUNSCRIPT do not exist, provide run script's path and name appropriately."
        err_flg=true
    fi

    if [[ ! -x $RS_OCS_INSTALL_PATH/bin/isql ]] ; then
        log_message 0 "ERROR: Executable - \"$RS_OCS_INSTALL_PATH/bin/isql\" does not exist"
        log_message 0 " - Make sure that isql is present in directory - $RS_OCS_INSTALL_PATH/bin"
        err_flg=true
    fi
fi # end of Non-Shared/Startup check 

# Confirm that User/uid exist on node
if ! $ID $RS_ADMIN_NAME >$NULL 2>&1 ; then
     log_message 0 "ERROR: RS admin user $RS_ADMIN_NAME does not exist"
     log_message 0 " - Add user $RS_ADMIN_NAME on the node ${HOST}."
     err_flg=true
fi

# Confirm that HEALTHCHK_CMD, an optional variable contains non-null string
if [[ -z "$RS_HEALTHCHK_CMD" && $OP_CODE != "stop" ]] ; then
    log_message 0 "WARNING: rs/HEALTHCHK_CMD is not defined in package configuration."
    log_message 0 " - Configure rs/HEALTHCHK_CMD appropriately, otherwise, health check will not be performed."
fi

# Confirm that RS_HEALTHCHK_RESULT variable contains non-null string
if [[ -n $RS_HEALTHCHK_CMD && -z "$RS_HEALTHCHK_RESULT" && $OP_CODE != "stop" ]] ; then
    log_message 0 "ERROR: rs/HEALTHCHK_RESULT is not defined for the health check command \"${RS_HEALTHCHK_CMD}\"."
    err_flg=true
elif [[ -n $RS_HEALTHCHK_CMD && -n $RS_HEALTHCHK_RESULT && $OP_CODE != "stop" ]] ; then
    log_message 0 "Configuring rs/HEALTHCHK_CMD:\"${RS_HEALTHCHK_CMD}\" and rs/HEALTHCHK_RESULT:\"${RS_HEALTHCHK_RESULT}\" " 
fi

if ${err_flg} ; then
   log_message 5 "RS Module verification failed..."
   return 1
fi

log_message 5 "RS Module verification success..." 
return 0
}

function get_rssd_name
{
    #get the configuration file for the RS
    rs_config=`$SED -n '/-C.* /{s/^.*\(-C.* \).*$/\1/;p;}' ${RS_RUNSCRIPT} | $SED 's/-C/ /g'` &&
    RSSD_SERVER_NAME=`$GREP "RSSD_server" $rs_config | $AWK -F"=" '{print $2}'`
    #get the RSSD_server through configuration file of RS
}

function start_rs
{

$PS -aef | $GREP ${RS_INSTANCE_NAME} | $GREP "repserver" | $GREP -v -w grep >$NULL 2>&1
if [[ $? -eq 0 ]] ; then
    log_message 0 "RS: RS instance ${RS_INSTANCE_NAME} is already started"
else
   $SU - ${RS_ADMIN_NAME} -c "/bin/sh" << EOF 
   if [[ -f ${RS_INSTALL_HOME}/SYBASE.sh ]] ; then
       . ${RS_INSTALL_HOME}/SYBASE.sh
       export PATH=$PATH:${RS_RS_INSTALL_PATH}/bin
       ${RS_RUNSCRIPT} >> ${RS_CONSOLE_LOGFILE} 2>&1 &
       $SLEEP ${RS_TIME_OUT}
   else
       $ECHO "ERROR: RS: Sybase environment variable file ${RS_INSTALL_HOME}/SYBASE.sh file is missing."
   fi #end of if SYBASE.sh file exist
EOF

## Check if process has started successfully - using ${RS_RETRY_TIME}

   $PS -aef | $GREP ${RS_INSTANCE_NAME} | $GREP "repserver" | $GREP -v -w grep >$NULL 2>&1
   if [[ $? -eq 0 ]] ; then
       log_message 0 "RS: RS instance ${RS_INSTANCE_NAME} is successfully started"
   else
       log_message 0 "ERROR: RS instance ${RS_INSTANCE_NAME} could not be started"
       log_message 0 " - Check console log file ${RS_CONSOLE_LOGFILE} or RSSD log file for possible error."
       return 1
   fi # end of if repserver successfully started

fi #end of if repserver already started 

return 0

}

function stop_rs
{
err_flag=false
declare -a RS_PID
typeset -i retry_kill=1

if ! ( $PS -efa | $GREP ${RS_INSTANCE_NAME} | $GREP "repserver" | $GREP -v -w grep >$NULL 2>&1 ) ; then
     log_message 0 "RS: RS process - \"repserver\" is already stopped/aborted."	
else
  if [[ -n ${RS_PASSWD_FILE} && -f ${RS_PASSWD_FILE} ]] ; then
   #get password from file
   if PASSWD=`$HEAD -n 1 ${RS_PASSWD_FILE} 2>$NULL | $AWK '{print $1}' 2>$NULL` ; then
      if [[ -z "${PASSWD}" ]] ; then
          log_message 0 "WARNING: RS: No password in file ${RS_PASSWD_FILE}."
      else
          $SU - ${RS_ADMIN_NAME} -c "/bin/sh" << EOF 
          if [[ -f ${RS_INSTALL_HOME}/SYBASE.sh ]] ; then
              . ${RS_INSTALL_HOME}/SYBASE.sh
              export PATH=$PATH:${RS_RS_INSTALL_PATH}/bin
${RS_OCS_INSTALL_PATH}/bin/isql -U${RS_USER_NAME} -P${PASSWD} -S${RS_INSTANCE_NAME} << SHUTDWN >$NULL   
shutdown
go
SHUTDWN
$SLEEP ${RS_TIME_OUT}
          else
	      $ECHO "ERROR: RS: Sybase environment variable file-${RS_INSTALL_HOME}/SYBASE.sh is missing."
          fi #end of if SYBASE.sh exists
EOF
      fi #end of PASSWD null
   else
      log_message 0 "WARNING: RS: Could not get password from file ${RS_PASSWD_FILE}."
   fi #end of get password 
  else 
     log_message 0 "WARNING: rs/PASSWD_FILE not configured or password file ${RS_PASSWD_FILE} does not exist"
     log_message 0 " - Sybase RS processes will be aborted."    
  fi #end of PASSWD_FILE configured/exist
fi #end repserver stopped

#get the name of RSSD server by calling get_rssd_name
   get_rssd_name

# Forceful shutdown
i=0
while [[ ${RS_MONITOR_PROCESS[@]} != "" ]]
do
# If shutdown was not successful; get rid of any existing processes by kill.
  case ${RS_MONITOR_PROCESS[${i}]} in
     "repserver")	
     RS_PID[${i}]=`$PS -efa | $GREP ${RS_INSTANCE_NAME} | $GREP ${RS_MONITOR_PROCESS[${i}]} | $GREP -v grep | $AWK '{print $2}' 2>$NULL`
     ;;

     "dbsrv"*)
     RS_PID[${i}]=`$PS -efa | $GREP ${RSSD_SERVER_NAME} | $GREP ${RS_MONITOR_PROCESS[${i}]} | $GREP -v grep | $AWK '{print $2}' 2>$NULL`
     ;;
  esac

  if [[ -n ${RS_PID[${i}]} ]] ; 
  then
      #If process was not gracefully shutdown...
      if $KILL -9 ${RS_PID[${i}]} ; then
         log_message 0 "RS: Success in killing ${RS_MONITOR_PROCESS[$i]}, PID: ${RS_PID[${i}]}"
         unset RS_MONITOR_PROCESS[$i]
         i=`expr $i + 1`
         retry_kill=1
      else
         if (( $retry_kill >= ${RS_RETRY_TIME} ))
         then
             log_message 0 "ERROR: RS: Could not abort process ${RS_MONITOR_PROCESS[$i]}, PID: ${RS_PID[${i}]}"
             sg_send_mail "${HOST}:${SG_PACKAGE}: Could not abort process - ${RS_MONITOR_PROCESS[$i]}" &&
             $SLEEP 3
             unset RS_MONITOR_PROCESS[$i]
             i=`expr $i + 1`
             err_flag=true
             retry_kill=1
         fi # end of if retry_kill -gt RS_RETRY_TIME
         retry_kill=`expr $retry_kill + 1`
      fi # end of if kill -9 pid
  else
      unset RS_MONITOR_PROCESS[$i]
      i=`expr $i + 1`
  fi #end of if RS_PID null
done #end of while 

#Kill dbltm process if exists; 
#dbltm process exists in multi RS environment and 
#this process is responsible for copying data across RS
DBLTM_PID=`$PS -aef | $GREP "dbltm" |  $GREP ${RSSD_SERVER_NAME} | $GREP -v -w grep | $AWK '{print $2}' 2>$NULL `
if [[ -n $DBLTM_PID ]] ; then
    if $KILL -9 $DBLTM_PID ; then
       log_message 0 "RS: Success in killing dbltm process, PID: $DBLTM_PID" 
    else
       err_flag=true
    fi
fi

if ${err_flag} ; then
	return 1
else
	log_message 0 "RS: All processes are stopped."
fi

return 0

}

##########################################################################
#
#	Re-configure Start/Stop functions
#
##########################################################################
# verify_service_restart
# return 0 - At least one attribute changed which needs service restart.
# return 1 - No attribute that needs restart has been changed.
# Based on, whether supported attribute changed which needs service restart
# on change, decide upon the restart of the service 
##########################################################################

function verify_service_restart
{
#Online re-configuration done to email attribute?
var1=( $(
     for id in "${SG_EMAIL_ID[@]}"
     do
       $ECHO "$id"
     done | $SORT ) )

var2=( $(
     for id in "${PREV_SG_EMAIL_ID[@]}"
     do
       $ECHO "$id"
     done | $SORT ) )

if [[ "${var1[@]}" != "${var2[@]}" ]] ; then
    return 0
fi

#Online re-configuration done to MONITOR_PROCESS ?
var3=( $(
     for proc in "${RS_MONITOR_PROCESS[@]}"
     do
        $ECHO "$proc"
     done | $SORT ) )

var4=( $(
     for proc in "${PREV_RS_MONITOR_PROCESS[@]}"
     do
       $ECHO "$proc"
     done | $SORT ) )

if [[ "${var3[@]}" != "${var4[@]}" ]] ; then
    return 0
fi

if [[ "$SG_SCRIPT_LOG_FILE" != "$PREV_SG_SCRIPT_LOG_FILE" ]] ; then
    return 0
fi
#Any other online reconfiguration supported attributes have been changed
#which needs restart of service?

if [[ "$RS_MONITOR_INTERVAL" != "$PREV_RS_MONITOR_INTERVAL" ]] ||
[[ "$RS_USER_NAME" != "$PREV_RS_USER_NAME" ]] ||
[[ "$RS_PASSWD_FILE" != "$PREV_RS_PASSWD_FILE" ]] ||
[[ "$RS_RETRY_TIME" != "$PREV_RS_RETRY_TIME" ]] ||
[[ "$RS_HEALTHCHK_CMD" != "$PREV_RS_HEALTHCHK_CMD" ]] ||
[[ "$RS_HEALTHCHK_RESULT" != "$PREV_RS_HEALTHCHK_RESULT" ]] ; then 
    return 0
fi

#There were no attribute re-configuration done 
#which would have required service restart

 log_message 5 "RS: None of the attribute that needs service restart have been modified online."
return 1
}


function reconfigure_stop_function
{
if ! sg_check_if_reconfig_has_failed ; then 
   if verify_service_restart ; then
      if ! ${SGSBIN}/cmhaltserv ${service_name} ; then
         sg_set_reconfig_failure
         return 1
      else
         log_message 999 "SUCCESS: Successfully stopped service: $service_name"
         log_message 0 "SUCCESS: Successfully stopped service: $service_name"
      fi
   fi
else
   sg_log 999 "WARNING: Not attempting to stop the service: ${service_name}"
fi

return 0
}

function reconfigure_start_function
{
   if verify_service_restart ; then
      msg="cmrunserv: Unable to start service $service_name (it is already active)"
      out=`${SGSBIN}/cmrunserv ${service_name} "${service_cmd_string}" 2>&1` 
      status=$?
      if [[ $status -ne 0 ]] && [[ "$msg" != "$out" ]] ; then
         log_message 999 "ERROR: Failed to start the service $service_name after online package modification"
         sg_set_reconfig_failure
         return 1
      elif [[ $status -eq 0 ]] ; then
         log_message 999 "SUCCESS: Successfully started service: $service_name"
            if ! sg_check_if_reconfig_has_failed ; then
               log_message 0 "SUCCESS: Successfully started service: $service_name"
            fi         
      fi
   fi
return 0
}

################################### MAIN #################################

OP_CODE=$1
REPSERV=true
CLUSTER_IP=${SG_IP_ADDRESS:-$HOST}
exit_val=0

log_message 5 "Module hars.sh called with argument(s): $*"

#Identify the service name and service command for the package
if [[ $OP_CODE = "reconfigure_stop" || $OP_CODE = "reconfigure_start" ]]; then
    if [[ ${#SG_SERVICE_CMD[@]} -ne 0 ]]; then
        i=0
        service_name=""
        service_cmd_string=""
        for cmd in "${SG_SERVICE_CMD[@]}"
        do
          $ECHO "$cmd" | $GREP -w "rs_monitor" >$NULL 2>&1
          if [[ $? -eq 0 ]]
          then
              service_name=${SG_SERVICE_NAME[$i]}
              service_cmd_string="$cmd"
              break
          fi
          i=`expr $i + 1`
        done
     fi
fi

case $OP_CODE in
   start)
           if sg_check_if_reconfig_has_failed ; then
              log_message 999 "WARNING: Not attempting to start Sybase RS module"
              return 
           else
              if ! start_rs ; then
                 if sg_check_if_it_is_reconfiguration ; then
                   log_message 999 "ERROR: Failed to start Sybase RS module"
                   sg_set_reconfig_failure
                 fi
                 check_return 1 1
              else
                 if sg_check_if_it_is_reconfiguration ; then
                    log_message 999 "SUCCESS: Successfully started Sybase RS module"
                 fi
              fi
           fi 
       ;;

   stop)
        if ! stop_rs ; then
             check_return 1 2
        fi
       ;;

   validate)
        if ! validate_rs ; then
             check_return 1 4
        fi
       ;;

   rs_monitor)
        . ${TKIT_SCRIPT_DIR}/hars.mon
	exit_val=$?
        check_return $exit_val 3
       ;;

   reconfigure_stop)
        if ! reconfigure_stop_function ; then
             log_message 0 "Service stop failed for - $service_name"
             exit 1
        fi
       ;;

   reconfigure_start)
        if ! reconfigure_start_function ; then
             log_message 0 "Service start failed for - $service_name" 
             exit 1
        fi
       ;;
   *)
        log_message 0 "${Usage_Msg}"
	exit_val=1
       ;;
esac

log_message 5 "End of hars.sh module"

exit ${exit_val}
